function [Territories,Alt_lists] = countries(Year, Lang)

%% COUNTRIES Generate a list of countries of the world.
%
%    [Territories,Alt_lists] = countries(YEAR,LANGUAGE) generates a text 
%    file  with list a of all independent states, dependent territories and
%    other political entities of the global system, divided by continent 
%    with a global summary. It has the following options:
%
%       YEAR            Year for which the list will be prepared (from 1800
%                       to 2017). 2017 is the default. Political situation
%                       is as of 31 December.
%       LANGUAGE        Optional. Language in which the list will be 
%                       prepared. Language can be 'EN' for English 
%                       (default) or 'PL' for Polish.
%
%    The function returns the following variables:
%    [Territories] -> a cell array with the territory code, 
%       territory name, status, year of latest population census and 
%       population at latest census.
%    [Alt_lists] -> a structure array containing 1-4 cell arrays with
%       other published lists of territories and their codes:
%       [GW] - list of independent states (population of 250,000 and more
%       plus a selection of smaller states) from Gleditsch, K.S. and 
%       Ward, M.D. (1999) A revised list of independent states since the 
%       congress of Vienna, International Interactions, 25(4), 393-413;
%       updated version available at:
%       http://ksgleditsch.com/statelist.html.
%       [PIV] - list of independent states (population of 500,000 and more)
%       from Polity IV Project (2018) Political Regime Characteristics and 
%       Transitions, 1800-2017. Available at:
%       http://www.systemicpeace.org/inscrdata.html
%       [COW] - list of independent states from Correlates of War Project
%       (2017) State System Membership List, v2016. Available at:
%       http://correlatesofwar.org
%       [ISO] - list of territories from International Organization for 
%       Standardization's standard ISO 3166-1:2013 "Codes for the 
%       representation of names of countries and their subdivisions -- 
%       Part 1: Country codes", reconstructed for 1974-2018 based on 
%       various Internet resources. Current version available at:
%       https://www.iso.org/obp/ui/#search
%

%% copyright
% (c) 2019 by Dominik Paprotny, GFZ German Research Centre for Geosciences.
% Use it at your own risk!

%% calculation

% if not args, assign defaults
if nargin<1
    Year = 2018; Lang = 'EN';
elseif nargin<2
    if isnumeric(Year)
        Lang = 'EN';
    else
        Lang = Year;
        Year = 2018;
    end
end
% Year check
if Year<1800
    error('Wrong year inserted. Function only valid for years 1800-2018')
elseif Year>2018
    error('Wrong year inserted. Function only valid for years 1800-2018')
else
end

load Countries_1800_2019.mat

Strtxt{1,1} = 'Lista pastw i terytoriw niesamodzielnych w roku %d\n\n';
Strtxt{1,2} = 'List of countries and dependent territories in %d\n\n';
Strtxt{2,1} = 'AFRYKA\n\n'; Strtxt{3,1} = 'AMERYKA PӣNOCNA \n\n'; Strtxt{4,1} = 'AMERYKA POUDNIOWA\n\n';  
Strtxt{5,1} = 'AZJA\n\n'; Strtxt{6,1} = 'EUROPA\n\n'; Strtxt{7,1} = 'OCEANIA\n\n'; Strtxt{8,1} = 'ANTARKTYKA\n\n';
Strtxt{2,2} = 'AFRICA\n\n'; Strtxt{3,2} = 'NORTH AMERICA\n\n'; Strtxt{4,2} = 'SOUTH AMERICA\n\n';
Strtxt{5,2} = 'ASIA\n\n'; Strtxt{6,2} = 'EUROPE\n\n'; Strtxt{7,2} = 'OCEANIA\n\n'; Strtxt{8,2} = 'ANTARCTICA\n\n';
Strtxt{9,1} = 'Pastwa bez powszechnego uznania midzynarodowego:\n';
Strtxt{9,2} = 'States with limited recognition:\n';
Strtxt{10,1} = ' (formalny suweren:';
Strtxt{10,2} = ' (formally part of';
Strtxt{11,1} = ' (okupant: ';
Strtxt{11,2} = ' (occupied by';
Strtxt{12,1} = ' i';
Strtxt{12,2} = ' and';
Strtxt{13,1} = 'Terytoria okupowane:\n';
Strtxt{13,2} = 'Occupied territories:\n';
if Year>=1920 && Year<1945
Strtxt{14,1} = 'Terytoria mandatowe Ligi Narodw:\n';
Strtxt{14,2} = 'League of Nations mandate territories:\n';
elseif Year>=1945
Strtxt{14,1} = 'Terytoria powiernicze ONZ:\n';
Strtxt{14,2} = 'United Nations trust territories:\n';
else
end
Strtxt{15,1} = 'Terytoria niesamodzielne:\n';
Strtxt{15,2} = 'Dependent territories:\n';
Strtxt{16,1} = 'Terytoria o nierozstrzygnitym statusie politycznym:\n';
Strtxt{16,2} = 'Territories with unresolved status:\n';
Strtxt{17,1} = ' (faktyczna kontrola: ';
Strtxt{17,2} = ' (controlled by';
Strtxt{18,1} = 'Niepodlege pastwa:\n';
Strtxt{18,2} = 'Independent states:\n';
Strtxt{19,1} = 'UWAGA: LISTA JEST NIEKOMPLETNA PRZED 1900 R.\n\n';
Strtxt{19,2} = 'NOTE: LIST IS INCOMPLETE BEFORE YEAR 1900\n\n';
Strtxt{20,1} = 'Ludno wiata (w milionach):\n %d \n';
Strtxt{20,2} = 'World population (million):\n %d \n';
Strtxt{21,1} = 'PODSUMOWANIE\n\n';
Strtxt{21,2} = 'SUMMARY\n\n';

if strcmp(Lang,'PL');
    Txts = Strtxt(:,1);
    Countries(:,5)=[];
    Names(:,3)=[];
else
    Txts = Strtxt(:,2);
    Countries(:,4)=[];
    Names(:,2)=[];
end

% generate country list
for i=1:length(Countries)
    if Countries{i,5}<=Year;
        if Countries{i,6}>Year;
            ia(i,1)=1;
        else
            ia(i,1)=0;
        end
    else
        ia(i,1)=0;
    end
end
ib = ismember(ia,1);
Countries2 = Countries(ib,:);
% Alt names list
for i=1:length(Names)
    if Names{i,3}<=Year;
        if Names{i,4}>Year;
            iaa(i,1)=1;
        else
            iaa(i,1)=0;
        end
    else
        iaa(i,1)=0;
    end
end
ib = ismember(iaa,1);
Names2 = Names(ib,:);
for j=1:length(Countries2)
    ie = ismember([Names2{:,1}],[Countries2{j,1}]);
    if sum(ie)==1
        Countries2{j,4} = Names2{ie,2};
    else
    end
end
% file and header
Filename = strcat('Countries_',num2str(Year),'_',Lang,'.txt');
fid = fopen(Filename,'wt');
fprintf(fid,Txts{1,1},Year);
if Year<1900
    fprintf(fid,Txts{19,1});
end
% list by continents
for i=1:7
    fprintf(fid,Txts{i+1,1});
    ic = ismember([Countries2{:,2}],i);
    Countries3 = Countries2(ic,:);
    % independent states
    id = ismember([Countries3{:,7}],1);
    Countries4 = Countries3(id,:);
    for j=1:size(Countries4,1)
        CN{j,1} = strcat(Countries4{j,4},'\n');
    end
    if size(j)==0;
    else
        CNT = sortrows(CN);
        for j=1:size(Countries4,1)
            fprintf(fid,CNT{j,1});
        end
    	clear CN CNT
        fprintf(fid,'\n');
    end
    %states with limited recognition
    id = ismember([Countries3{:,7}],2);
    Countries4 = Countries3(id,:);
    for j=1:size(Countries4,1)
        ig1 = ismember([Countries2{:,1}],[Countries4{j,8}]);
        CN{j,1} = strcat('\t',Countries4{j,4},Txts{10,1},{' '},Countries2{ig1,4},')\n');
    end
    if size(j)==0;
    else
        fprintf(fid,Txts{9,1});
        CNT = sortrows([CN{:}]');
        for j=1:size(Countries4,1)
            fprintf(fid,CNT{j,1});
        end
    	clear CN CNT
        fprintf(fid,'\n');
    end
    %occupied territories
    id = ismember([Countries3{:,7}],[3, 6]);
    Countries4 = Countries3(id,:);
    for j=1:size(Countries4,1)
        ig1 = ismember([Countries2{:,1}],[Countries4{j,8}]);
        ig2 = ismember([Countries2{:,1}],[Countries4{j,9}]);
        ig3 = ismember([Countries2{:,1}],[Countries4{j,10}]);
        if sum(ig3)==1;
            CN{j,1} = strcat('\t',Countries4{j,4},Txts{11,1},{' '},Countries2{ig1,4},{', '},Countries2{ig2,4},Txts{12,1},{' '},Countries2{ig3,4},')\n');
        elseif sum(ig2)==1;
            CN{j,1} = strcat('\t',Countries4{j,4},Txts{11,1},{' '},Countries2{ig1,4},Txts{12,1},{' '},Countries2{ig2,4},')\n');
        else
            CN{j,1} = strcat('\t',Countries4{j,4},Txts{11,1},{' '},Countries2{ig1,4},')\n');
        end
    end
    if size(j)==0;
    else
        fprintf(fid,Txts{13,1});
        CNT = sortrows([CN{:}]');
        for j=1:size(Countries4,1)
            fprintf(fid,CNT{j,1});
        end
    	clear CN CNT
        fprintf(fid,'\n');
    end
    %mandate territories
    id = ismember([Countries3{:,7}], 4);
    Countries4 = Countries3(id,:);
    for j=1:size(Countries4,1)
        ig1 = ismember([Countries2{:,1}],[Countries4{j,8}]);
        ig2 = ismember([Countries2{:,1}],[Countries4{j,9}]);
        ig3 = ismember([Countries2{:,1}],[Countries4{j,10}]);
        if sum(ig3)==1;
            CN{j,1} = strcat({'\t\t'},Countries4{j,4},{'\n'});
            CW{j,1} = strcat({'\t'},Countries2{ig1,4},{', '},Countries2{ig2,4},Txts{12,1},{' '},Countries2{ig3,4},':\n');
        elseif sum(ig2)==1;
            CN{j,1} = strcat({'\t\t'},Countries4{j,4},{'\n'});
            CW{j,1} = strcat({'\t'},Countries2{ig1,4},Txts{12,1},{' '},Countries2{ig2,4},':\n');
        else
            CN{j,1} = strcat({'\t\t'},Countries4{j,4},{'\n'});
            CW{j,1} = strcat({'\t'},Countries2{ig1,4},{':\n'});
        end
    end
    if size(j)==0;
    else
        fprintf(fid,Txts{14,1});
        MAN = unique([CW{:}]');
        MANT = sortrows(MAN);
        for l=1:size(MANT,1)
            fprintf(fid,MANT{l,1});
            ih = ismember([CW{:}],MANT{l,1})';
            CNT = CN(ih);
            CNT = sortrows([CNT{:}]');
            for j=1:size(CNT,1)
                fprintf(fid,CNT{j,1});
            end
        end
    	clear CN CW CNT
        fprintf(fid,'\n');
    end
    %dependent territories
    id = ismember([Countries3{:,7}], 5);
    Countries4 = Countries3(id,:);
    for j=1:size(Countries4,1)
        ig1 = ismember([Countries2{:,1}],[Countries4{j,8}]);
        ig2 = ismember([Countries2{:,1}],[Countries4{j,9}]);
        ig3 = ismember([Countries2{:,1}],[Countries4{j,10}]);
        if sum(ig3)==1;
            CN{j,1} = strcat({'\t\t'},Countries4{j,4},{'\n'});
            CW{j,1} = strcat({'\t'},Countries2{ig1,4},{', '},Countries2{ig2,4},Txts{12,1},{' '},Countries2{ig3,4},':\n');
        elseif sum(ig2)==1;
            CN{j,1} = strcat({'\t\t'},Countries4{j,4},{'\n'});
            CW{j,1} = strcat({'\t'},Countries2{ig1,4},Txts{12,1},{' '},Countries2{ig2,4},':\n');
        else
            CN{j,1} = strcat({'\t\t'},Countries4{j,4},{'\n'});
            CW{j,1} = strcat({'\t'},Countries2{ig1,4},{':\n'});
        end
    end
    if size(j)==0;
    else
        fprintf(fid,Txts{15,1});
        MAN = unique([CW{:}]');
        MANT = sortrows(MAN);
        for l=1:size(MANT,1)
            fprintf(fid,MANT{l,1});
            ih = ismember([CW{:}],MANT{l,1})';
            CNT = CN(ih);
            CNT = sortrows([CNT{:}]');
            for j=1:size(CNT,1)
                fprintf(fid,CNT{j,1});
            end
        end
    	clear CN CW CNT
        fprintf(fid,'\n');
    end
    %status unresolved
    id = ismember([Countries3{:,7}],8);
    Countries4 = Countries3(id,:);
    for j=1:size(Countries4,1)
        ig1 = ismember([Countries2{:,1}],[Countries4{j,8}]);
        ig2 = ismember([Countries2{:,1}],[Countries4{j,9}]);
        ig3 = ismember([Countries2{:,1}],[Countries4{j,10}]);
        if sum(ig3)==1;
            CN{j,1} = strcat('\t',Countries4{j,4},Txts{17,1},{' '},Countries2{ig1,4},{', '},Countries2{ig2,4},Txts{12,1},{' '},Countries2{ig3,4},')\n');
        elseif sum(ig2)==1;
            CN{j,1} = strcat('\t',Countries4{j,4},Txts{17,1},{' '},Countries2{ig1,4},Txts{12,1},{' '},Countries2{ig2,4},')\n');
        else
            CN{j,1} = strcat('\t',Countries4{j,4},Txts{17,1},{' '},Countries2{ig1,4},')\n');
        end
    end
    if size(j)==0;
    else
        fprintf(fid,Txts{16,1});
        CNT = sortrows([CN{:}]');
        for j=1:size(Countries4,1)
            fprintf(fid,CNT{j,1});
        end
    	clear CN CNT
        fprintf(fid,'\n');
    end
end
fprintf(fid,'\n');
fprintf(fid,Txts{21,1});
WP = round(Population(Population(:,1)==Year,2)/1000);
fprintf(fid,Txts{20,1},WP);
ix = ismember([Countries2{:,7}],1);
RS = strcat(Txts{18,1},' %d \n');
fprintf(fid,RS,sum(ix));
ix = ismember([Countries2{:,7}],2);
RS = strcat(Txts{9,1},' %d \n');
fprintf(fid,RS,sum(ix));
ix = ismember([Countries2{:,7}],[3,6]);
RS = strcat(Txts{13,1},' %d \n');
fprintf(fid,RS,sum(ix));
if Year>=1920
ix = ismember([Countries2{:,7}],4);
RS = strcat(Txts{14,1},' %d \n');
fprintf(fid,RS,sum(ix));
else
end
ix = ismember([Countries2{:,7}],5);
RS = strcat(Txts{15,1},' %d \n');
fprintf(fid,RS,sum(ix));
ix = ismember([Countries2{:,7}],8);
RS = strcat(Txts{16,1},' %d \n');
fprintf(fid,RS,sum(ix));
ix = ismember([Countries2{:,7}],[1 2 3 4 5 6 8]);
%Territories = sum(ix);
Countries5 = Countries2(ix,[1 4 7]);
for k=1:size(Countries5,1)
    iy = ismember([Status{:,1}],[Countries5{k,3}]);
    if strcmp(Lang,'PL');
        Countries5(k,3) = Status(iy,2);
    else
        Countries5(k,3) = Status(iy,3);
    end
end
Territories = sortrows(Countries5,2);
%List_GW = sum(ix);
if Year<1816
else
for i=1:length(List_GW)
    if List_GW{i,5}<=Year;
        if List_GW{i,6}>Year;
            ba(i,1)=1;
        else
            ba(i,1)=0;
        end
    else
        ba(i,1)=0;
    end
end
bb = ismember(ba,1);
Alt_lists(1).GW = List_GW(bb,1:4);
clear ba bb
end
%List_PIV = sum(ix);
for i=1:length(List_PIV)
    if List_PIV{i,5}<=Year;
        if List_PIV{i,6}>Year;
            ba(i,1)=1;
        else
            ba(i,1)=0;
        end
    else
        ba(i,1)=0;
    end
end
bb = ismember(ba,1);
Alt_lists(1).PIV = List_PIV(bb,[1:4 8]);
clear ba bb
%List_COW = sum(ix);
if Year<1816
else
for i=1:length(List_COW)
    if List_COW{i,5}<=Year;
        if List_COW{i,6}>Year;
            ba(i,1)=1;
        else
            ba(i,1)=0;
        end
    else
        ba(i,1)=0;
    end
end
bb = ismember(ba,1);
Alt_lists(1).COW = List_COW(bb,1:4);
clear ba bb
end
%List_ISO = sum(ix);
if Year<1974
else
for i=1:length(List_ISO)
    if List_ISO{i,6}<=Year;
        if List_ISO{i,7}>Year;
            ba(i,1)=1;
        else
            ba(i,1)=0;
        end
    else
        ba(i,1)=0;
    end
end
bb = ismember(ba,1);
Alt_lists(1).ISO = List_ISO(bb,1:5);
clear ba bb
end
% census data
load('Census_data.mat')
cc = cell2mat(Census(:,[1 13 14]));
icc = cc(:,2)<=Year&cc(:,3)>=Year;
Census2 = Census(icc,:);
Territories{1,4}=[];
Territories{1,5}=[];
for i=1:size(Territories,1)
    ic2 = cc(icc,1)==Territories{i,1};
    if sum(ic2)==0
    else
    Territories(i,4)=Census2(ic2,5);
    Territories(i,5)=Census2(ic2,8);
    end
end
% close
s = fclose(fid);